@@ -915,26 +915,23 @@ def _prepareCleaningSelectors(args):
 
     :arg args: ``argparse.Namespace`` object.
     """
-    def numlist(s):
-        return np.fromstring(s, sep=',', dtype=np.float)
 
-    # convert na_values from a sequence of [(varid, str)]
-    # pairs into a dict of {varid : [value]} mappings
+    # convert na_values from a sequence of [(varid, values)]
+    # pairs into a dict of {varid : values} mappings
     if args.na_values is not None:
-        args.na_values = {int(vid) : numlist(values)
-                          for vid, values in args.na_values}
+        args.na_values = {int(vid) : values for vid, values in args.na_values}
 
     # Convert recoding from a sequence of [(varid, rawlevels, newlevels)]
     # tuples to a dict of {varid : (rawlevels, newlevels)} mappings
     if args.recoding is not None:
-        args.recoding = {int(vid) : (numlist(rawlevels), numlist(newlevels))
+        args.recoding = {int(vid) : (rawlevels, newlevels)
                          for vid, rawlevels, newlevels in args.recoding}
 
     # Convert child_values from a sequence of
     # [(varid, exprs, values)] tuples to a dict of
-    # {varid : ([exprs], [values])} mappings
+    # {varid : (exprs, values)} mappings
     if args.child_values is not None:
-        args.child_values = {int(vid) : (exprs, numlist(values))
+        args.child_values = {int(vid) : (exprs, values)
                              for vid, exprs, values in args.child_values}
 
     # convert clean from a sequence of [(varid, expr)]
