@@ -14,7 +14,8 @@ import os.path   as op
 import              os
 import              logging
 
-import numpy     as np
+import numpy            as np
+import pandas.api.types as pdtypes
 
 from . import util
 from . import custom
@@ -50,6 +51,7 @@ def exportTSV(dtable,
               outfile,
               sep=None,
               missingValues=None,
+              escapeNewlines=False,
               numRows=None,
               dropNaRows=False,
               dateFormat=None,
@@ -71,6 +73,10 @@ def exportTSV(dtable,
     :arg missingValues:  String to use for missing/NA values. Defaults to the
                          empty string.
 
+
+    :arg escapeNewlines: If ``True``, all string/object types are escaped
+                         using ``shlex.quote``.
+
     :arg numRows:        Number of rows to write at a time. Defaults to
                          :attr:`NUM_ROWS`.
 
@@ -89,6 +95,7 @@ def exportTSV(dtable,
     if sep           is None: sep           = '\t'
     if missingValues is None: missingValues = ''
     if numRows       is None: numRows       = NUM_ROWS
+    if formatters    is None: formatters    = {}
 
     # We're going to output each chunk of
     # subjects to a separate file (in
@@ -101,6 +108,17 @@ def exportTSV(dtable,
     subtables  = [dtable.subtable(rows=c) for c in idxchunks]
     outfiles   = ['{}_{}'.format(outfile, i) for i in range(nchunks)]
 
+    # escapeNewlines is performed by the
+    # exporting.escapeString formatter
+    # function. We apply it to all columns
+    # which are not numeric or date
+    if escapeNewlines:
+        for col in dtable.dataColumns:
+            series = dtable[:, col.name]
+            if not (pdtypes.is_numeric_dtype(       series) or
+                    pdtypes.is_datetime64_any_dtype(series)):
+                formatters[col.name] = 'escapeString'
+
     # write each chunk in parallel
     args = zip(subtables,
                outfiles,
