@@ -71,6 +71,7 @@ been evaluated on multiple columns to combine the results column-wise.
 
 import                    collections
 import                    logging
+import                    re
 import collections.abc as abc
 import itertools       as it
 import functools       as ft
@@ -171,8 +172,10 @@ def calculateExpressionEvaluationOrder(vids, exprs):
                 expression function, or a sequence of them.
 
     :returns:   A list of tuples, each containing:
+
                  - A hierarchy level
                  - A list of all variables at that level
+
                 The list is in ascending order, by the hierarchy level
     """
 
@@ -319,27 +322,31 @@ def makeParser():
     if getattr(makeParser, 'parser', None) is not None:
         return makeParser.parser
 
-    CMP     = ['eq', 'ne', 'lt', 'le', 'gt', 'ge']
-    CMPOP   = pp.oneOf([SYMBOLS[c] for c in CMP])
-    EQOP    = pp.oneOf([SYMBOLS[c] for c in ['eq', 'ne']])
-    STROP   = pp.oneOf([SYMBOLS[c] for c in ['eq', 'ne', 'contains']])
-    ANY     = pp.CaselessLiteral(SYMBOLS['any'])
-    ALL     = pp.CaselessLiteral(SYMBOLS['all'])
-    AND     = pp.CaselessLiteral(SYMBOLS['and'])
-    OR      = pp.CaselessLiteral(SYMBOLS['or'])
-    NOT     = pp.CaselessLiteral(SYMBOLS['not'])
-    NA      = pp.CaselessLiteral(SYMBOLS['na'])
-    NUM     = pp.pyparsing_common.number
-    STR     = pp.QuotedString("'") ^ pp.QuotedString('"')
-    VAR     = (pp.CaselessLiteral(SYMBOLS['var']) +
-               pp.pyparsing_common.integer).setParseAction(parseVariable)
+    CMP   = ['eq', 'ne', 'lt', 'le', 'gt', 'ge']
+    CMPOP = pp.oneOf([SYMBOLS[c] for c in CMP])
+    EQOP  = pp.oneOf([SYMBOLS[c] for c in ['eq', 'ne']])
+    STROP = pp.oneOf([SYMBOLS[c] for c in ['eq', 'ne', 'contains']])
+    ANY   = pp.CaselessLiteral(SYMBOLS['any'])
+    ALL   = pp.CaselessLiteral(SYMBOLS['all'])
+    AND   = pp.CaselessLiteral(SYMBOLS['and'])
+    OR    = pp.CaselessLiteral(SYMBOLS['or'])
+    NOT   = pp.CaselessLiteral(SYMBOLS['not'])
+    NA    = pp.CaselessLiteral(SYMBOLS['na'])
+    NUM   = pp.pyparsing_common.number
+    DATE  = pp.Regex(r'\d\d\d\d-\d\d-\d\d',                re.ASCII)
+    TIME  = pp.Regex(r'\d\d\d\d-\d\d-\d\d \d\d:\d\d:\d\d', re.ASCII)
+    DATE  = DATE ^ TIME
+    STR   = pp.QuotedString("'") ^ pp.QuotedString('"')
+    VAR   = (pp.CaselessLiteral(SYMBOLS['var']) +
+             pp.pyparsing_common.integer).setParseAction(parseVariable)
 
     # a single conditional statement:
     # "variable comparison_operator value"
-    NUMCOND = pp.Group(VAR + CMPOP + NUM).setParseAction(parseCondition)
-    STRCOND = pp.Group(VAR + STROP + STR).setParseAction(parseCondition)
-    NACOND  = pp.Group(VAR + EQOP  + NA) .setParseAction(parseCondition)
-    COND    = NUMCOND ^ STRCOND ^ NACOND
+    NUMCOND  = pp.Group(VAR + CMPOP + NUM) .setParseAction(parseCondition)
+    STRCOND  = pp.Group(VAR + STROP + STR) .setParseAction(parseCondition)
+    DATECOND = pp.Group(VAR + CMPOP + DATE).setParseAction(parseCondition)
+    NACOND   = pp.Group(VAR + EQOP  + NA)  .setParseAction(parseCondition)
+    COND     = NUMCOND ^ STRCOND ^ DATECOND ^ NACOND
 
     # the infixNotation helper does the heavy
     # lifting for boolean/combine operations
