@@ -19,15 +19,13 @@ import                          collections
 
 import pandas                as pd
 import numpy                 as np
-
-from .. import                  util
-from .. import                  custom
-from .. import                  merging
-from .. import                  fileinfo
-from .. import                  datatable
-from .. import                  loadtables
-from .  import                  filter   # noqa
-from .  import                  reindex
+import funpack.util          as util
+import funpack.custom        as custom
+import funpack.merging       as merging
+import funpack.datatable     as datatable
+import funpack.loadtables    as loadtables
+from . import                   filter   # noqa
+from . import                   reindex
 
 
 log = logging.getLogger(__name__)
@@ -62,7 +60,7 @@ function.
 """
 
 
-def importData(datafiles,
+def importData(fileinfo,
                vartable,
                proctable,
                cattable,
@@ -72,14 +70,11 @@ def importData(datafiles,
                subjects=None,
                subjectExprs=None,
                exclude=None,
-               encoding=None,
                trustTypes=False,
-               indexes=None,
                mergeAxis=None,
                mergeStrategy=None,
                indexVisits=False,
                dropNaRows=False,
-               loaders=None,
                njobs=1,
                mgr=None,
                dryrun=False):
@@ -94,7 +89,7 @@ def importData(datafiles,
 
       3. Creates and returns a :class:`DataTable`.
 
-    :arg datafiles:     Path to the data file(s)
+    :arg fileinfo:      :class:`.FileInfo` object describing the input file(s).
 
     :arg vartable:      The data coding table
 
@@ -115,18 +110,11 @@ def importData(datafiles,
 
     :arg exclude:       List of subjects to exclude
 
-    :arg encoding:      Character encoding(s) for data file(s). See
-                        :func:`loadData`.
-
     :arg trustTypes:    If ``True``, it is assumed that columns with a
                         known data type do not contain any bad/unparseable
                         values. This improves performance, but will cause
                         an error if the assumption does not hold.
 
-    :arg indexes:       Dict of ``{filename : [index]}`` mappings,
-                        specifying the position of the column(s) to use as
-                        the index. Defaults to 0 (the first column).
-
     :arg mergeAxis:     Merging axis to use when loading multiple data
                         files - see the :func:`mergeData` function.
 
@@ -141,10 +129,6 @@ def importData(datafiles,
     :arg dropNaRows:    If ``True``, rows which do not contain data for any
                         columns are not loaded.
 
-    :arg loaders:       Dict of ``{ file : loaderName }`` mappings
-                        containing custom sniffers/loaders to be used for
-                        specific files. See the :mod:`.custom` module.
-
     :arg njobs:         Number of processes to use for parallelising tasks.
 
     :arg mgr:           :class:`multiprocessing.Manager` object for
@@ -165,12 +149,10 @@ def importData(datafiles,
     variables = filter.restrictVariables(cattable, variables, categories)
 
     # Figure out which columns to load
-    cols, drop = filter.columnsToLoad(datafiles,
+    cols, drop = filter.columnsToLoad(fileinfo,
                                       vartable,
                                       variables,
-                                      colnames,
-                                      indexes=indexes,
-                                      sniffers=loaders)
+                                      colnames)
 
     # Load those columns, merging
     # multiple input files.
@@ -178,19 +160,16 @@ def importData(datafiles,
     else:         Pool = mpd.Pool
 
     with Pool(njobs) as pool:
-        data, cols = loadFiles(datafiles,
+        data, cols = loadFiles(fileinfo,
                                vartable,
                                cols,
                                subjects=subjects,
                                subjectExprs=subjectExprs,
                                exclude=exclude,
-                               encoding=encoding,
-                               indexes=indexes,
                                mergeAxis=mergeAxis,
                                mergeStrategy=mergeStrategy,
                                indexVisits=indexVisits,
                                dropNaRows=dropNaRows,
-                               loaders=loaders,
                                trustTypes=trustTypes,
                                pool=pool,
                                dryrun=dryrun)
@@ -233,27 +212,24 @@ def importData(datafiles,
     return dtable, drop
 
 
-def loadFiles(datafiles,
+def loadFiles(fileinfo,
               vartable,
               columns,
               nrows=None,
               subjects=None,
               subjectExprs=None,
               exclude=None,
-              encoding=None,
-              indexes=None,
               trustTypes=False,
               mergeAxis=None,
               mergeStrategy=None,
               indexVisits=False,
               dropNaRows=False,
-              loaders=None,
               pool=None,
               dryrun=False):
     """Load data from ``datafiles``, using :func:`.mergeDataFrames` if multiple
     files are provided.
 
-    :arg datafiles:     Path to the data file(s)
+    :arg fileinfo:      :class:`.FileInfo` object describing the input file(s).
 
     :arg vartable:      Variable table
 
@@ -270,13 +246,6 @@ def loadFiles(datafiles,
 
     :arg exclude:       List of subjects to exclude
 
-    :arg encoding:      Character encoding (or sequence of encodings, one
-                        for each data file). Defaults to ``latin1``.
-
-    :arg indexes:       Dict of ``{filename : [index]}`` mappings, specifying
-                        the position of the column(s) to use as the index.
-                        Defaults to 0 (the first column).
-
     :arg trustTypes:    Assume that columns with known data type do not contain
                         any bad/unparseable values.
 
@@ -296,10 +265,6 @@ def loadFiles(datafiles,
     :arg dropNaRows:    If ``True``, rows which do not contain data for any
                         columns are not loaded.
 
-    :arg loaders:       Dict of ``{ file : loaderName }`` mappings containing
-                        custom loaders/sniffers to be used for specific files.
-                        See the :mod:`.custom` module.
-
     :arg pool:          ``multiprocessing.Pool`` object for running tasks in
                         parallel.
 
@@ -315,27 +280,14 @@ def loadFiles(datafiles,
 
     if mergeStrategy is None: mergeStrategy = MERGE_STRATEGY
     if mergeAxis     is None: mergeAxis     = MERGE_AXIS
-    if loaders       is None: loaders       = {}
-    if indexes       is None: indexes       = {}
-
-    if isinstance(datafiles, str):
-        datafiles = [datafiles]
-    if encoding is None or isinstance(encoding, str):
-        encoding = [encoding] * len(datafiles)
-
-    # Get the format for each input file
-    dialects, headers, names = fileinfo.fileinfo(
-        datafiles, indexes, loaders)
 
     # load the data
     data       = []
     loadedcols = []
-    for fname, fencoding, dialect, header, allcols in zip(
-            datafiles, encoding, dialects, headers, names):
+    for fname in fileinfo.datafiles:
 
-        toload = columns[fname]
-        loader = loaders.get(fname, None)
-        index  = indexes.get(fname, [0])
+        toload = columns[        fname]
+        loader = fileinfo.loader(fname)
         fdata  = None
         fcols  = None
 
@@ -347,19 +299,15 @@ def loadFiles(datafiles,
         else:
             log.debug('Loading %s with pandas', fname)
             fdata, fcols = loadFile(fname,
+                                    fileinfo,
                                     vartable,
-                                    header,
-                                    dialect,
-                                    allcols,
                                     toload,
-                                    index=index,
                                     nrows=nrows,
                                     subjects=subjects,
                                     subjectExprs=subjectExprs,
                                     exclude=exclude,
                                     indexVisits=indexVisits,
                                     dropNaRows=dropNaRows,
-                                    encoding=fencoding,
                                     trustTypes=trustTypes,
                                     pool=pool)
 
@@ -396,19 +344,15 @@ def loadFiles(datafiles,
 
 
 def loadFile(fname,
+             fileinfo,
              vartable,
-             header,
-             dialect,
-             allcols,
              toload,
-             index=None,
              nrows=None,
              subjects=None,
              subjectExprs=None,
              exclude=None,
              indexVisits=False,
              dropNaRows=False,
-             encoding=None,
              trustTypes=False,
              pool=None):
     """Loads data from the specified file. The file is loaded in chunks of
@@ -416,23 +360,14 @@ def loadFile(fname,
 
     :arg fname:        Path to the data file
 
+    :arg fileinfo:     :class:`.FileInfo` object describing the input file.
+
     :arg vartable:     Variable table
 
-    :arg header:       ``True`` if the file has a header row, ``False``
-                       otherwise.
-
-    :arg dialect:      File dialect (see :func:`.fileinfo`).
-
-    :arg allcols:      Sequence of :class:`.Column` objects describing all
-                       columns in the file.
-
     :arg toload:       Sequence of :class:`.Column` objects describing the
                        columns that should be loaded, as generated by
                        :func:`columnsToLoad`.
 
-    :arg index:        List containing position(s) of index column(s).
-                       Defaults to [0].
-
     :arg nrows:        Number of rows to read at a time. Defaults to
                        attr:`NUM_ROWS`.
 
@@ -450,9 +385,6 @@ def loadFile(fname,
     :arg dropNaRows:   If ``True``, rows which do not contain data for any
                        columns are not loaded.
 
-    :arg encoding:     Character encoding (or sequence of encodings, one
-                       for each data file). Defaults to ``latin1``.
-
     :arg trustTypes:   Assume that columns with known data type do not contain
                        any bad/unparseable values.
 
@@ -460,6 +392,7 @@ def loadFile(fname,
                        parallel.
 
     :returns:          A tuple containing:
+
                         - A ``pandas.DataFrame`` containing the data.
                         - A list of :class:`.Column` objects representing the
                           columns that were loaded. Note that this may be
@@ -470,10 +403,8 @@ def loadFile(fname,
     ownPool = pool is None
     toload  = list(toload)
 
-    if index    is None: index    = [0]
-    if encoding is None: encoding = 'latin1'
-    if nrows    is None: nrows    = NUM_ROWS
-    if pool     is None: pool     = mpd.Pool(1)
+    if nrows is None: nrows = NUM_ROWS
+    if pool  is None: pool  = mpd.Pool(1)
 
     # The read_csv function requires the
     # index argument to be specified
@@ -485,6 +416,7 @@ def loadFile(fname,
     #
     # So here we make index relative to
     # toload.
+    index = fileinfo.index(fname)
     index = [i for i, c in enumerate(toload) if c.index in index]
 
     # Figure out suitable data types to
@@ -510,8 +442,8 @@ def loadFile(fname,
 
     # input may or may not
     # have a header row
-    if header: header = 0
-    else:      header = None
+    if fileinfo.header(fname): header = 0
+    else:                      header = None
 
     if indexVisits:
         indexVisits = reindex.genReindexedColumns(toload, vartable)
@@ -523,6 +455,10 @@ def loadFile(fname,
 
     # Prepare arguments to
     # the loadChunk function
+    allcols  = fileinfo.columns( fname)
+    dialect  = fileinfo.dialect( fname)
+    encoding = fileinfo.encoding(fname)
+
     args = {'fname'        : fname,
             'vartable'     : vartable,
             'header'       : header,
@@ -660,6 +596,9 @@ def loadChunk(i,
     :returns:          ``pandas.DataFrame``
     """
 
+    if encoding is None:
+        encoding = 'latin1'
+
     allcolnames = [c.name for c in allcols]
     toloadnames = [c.name for c in toload]
 
