@@ -319,7 +319,8 @@ def splitDataTable(dtable, args):
     # if suppress, only numeric columns
     # are saved to main output file
     if args.suppress_non_numerics:
-        log.debug('Separating out %u numeric columns for export', len(ncols))
+        log.debug('Separating out %u / %u numeric columns for export',
+                  len(ncols), len(dtable.dataColumns))
         dtables.append((dtable.subtable(ncols), args.outfile))
     else:
         dtables.append((dtable, args.outfile))
@@ -327,7 +328,8 @@ def splitDataTable(dtable, args):
     # if write, non-numeric columns
     # are saved to an auxillary file
     if args.write_non_numerics:
-        log.debug('Separating out %u non-numeric columns for export', len(nncols))
+        log.debug('Separating out %u / %u non-numeric columns for export',
+                  len(nncols), len(dtable.dataColumns))
         dtables.append((dtable.subtable(nncols), args.non_numerics_file))
 
     return dtables
@@ -519,10 +521,10 @@ def doSummaryExport(dtable, args):
 
     vartable = dtable.vartable
     vids     = sorted(dtable.variables)[1:]
-    sumdf    = pd.DataFrame(columns=['ID', 'NAValues',
-                                     'RawLevels', 'NewLevels',
+    sumdf    = pd.DataFrame(columns=['NAValues', 'RawLevels', 'NewLevels',
                                      'ParentValues', 'ChildValues',
-                                     'Clean', 'Flags']).set_index('ID')
+                                     'Clean', 'Flags'], index=vids)
+    sumdf.index.name = 'ID'
 
     with util.timed('Summary export', log):
         for vid in vids:
