@@ -78,9 +78,9 @@ the following:
        all_independent_except,1,2,3 processName
 
 
-For example, the :func:`.binariseCategorical` function applies its
-logic independently to each variable, so it makes sense to specify that it
-should be applied independently to a set of variables::
+For example, the :func:`.processing_functions.binariseCategorical` function
+applies its logic independently to each variable, so it makes sense to specify
+that it should be applied independently to a set of variables::
 
     independent,1,2,3,4,5 binariseCategorical
 
@@ -99,9 +99,9 @@ arguments to the process may need to be different for each variable. This can
 be accomplished by using a _broadcast_ argument - simply prefix the argument
 name with ``'broadcast_'``, and then specify a list containing the argument
 values for each variable. For example, the following specification will result
-in the :func:`.binariseCategorical` process being applied independently to
-variables 1, 2, and 3, with values taken from variables 4, 5, and 6
-respectively.
+in the :func:`.processing_functions.binariseCategorical` process being applied
+independently to variables 1, 2, and 3, with values taken from variables 4, 5,
+and 6 respectively.
 
     independent,1,2,3 binariseCategorical(broadcast_take=[4, 5, 6])
 
@@ -144,8 +144,8 @@ def processData(dtable):
 
     for i in dtable.proctable.index:
 
-        procs, vids, parallel, bcastIdxs = retrieveProcess(dtable, i)
-        allvids                          = list(it.chain(*vids))
+        procs, vids, parallel = retrieveProcess(dtable, i)
+        allvids               = list(it.chain(*vids))
 
         if len(allvids) == 0:
             continue
@@ -163,7 +163,7 @@ def processData(dtable):
 
             with util.timed(proc.name, log, logging.DEBUG, fmt=fmt), \
                  tempfile.TemporaryDirectory() as workDir:
-                runProcess(proc, dtable, vids, workDir, parallel, bcastIdxs)
+                runProcess(proc, dtable, vids, workDir, parallel)
 
 
 def retrieveProcess(dtable, procIdx):
@@ -172,7 +172,7 @@ def retrieveProcess(dtable, procIdx):
     process should be applied to.
 
     :arg dtable:  The :class:`.DataTable`
-    :arg procIdx: Index into the processing  table
+    :arg procIdx: Index into the processing table
     :returns:     A tuple containing:
 
                     - A dict of ``{ name : Process }`` mappings containing
@@ -183,17 +183,12 @@ def retrieveProcess(dtable, procIdx):
 
                     - ``True`` if the process can be applied in parallel
                       across the variable groups, ``False`` otherwise.
-
-                    - A list of indices specifying the broadcast arguments
-                      for each variable group, or ``None`` if no broadcasting
-                      can take place.
     """
 
     i         = procIdx
     ptable    = dtable.proctable
     all_vids  = dtable.variables
     all_vids  = [v for v in all_vids if v != 0]
-    bcastIdxs = None
 
     # For each process, the processing table
     # contains a "process variable type",
@@ -230,18 +225,16 @@ def retrieveProcess(dtable, procIdx):
 
     # apply independently to specified variables
     elif pvtype == 'independent':
-        vids      = [([v], i) for i, v in enumerate(vids) if dtable.present(v)]
-        bcastIdxs = [v[1] for v in vids]
-        vids      = [v[0] for v in vids]
+        vids = [[v] for v in vids]
 
     # apply simultaneously to specified variables
-    else:  # 'vids'
-        vids = [[v for v in vids if dtable.present(v)]]
+    else:  # ptype == 'vids'
+        vids = [vids]
 
-    return procs, vids, 'independent' in pvtype, bcastIdxs
+    return procs, vids, 'independent' in pvtype
 
 
-def runProcess(proc, dtable, vids, workDir, parallel, bcastIdxs):
+def runProcess(proc, dtable, vids, workDir, parallel):
     """Called by :func:`processData`. Runs the given process, and updates
     the :class:`.DataTable` as needed.
 
@@ -253,29 +246,64 @@ def runProcess(proc, dtable, vids, workDir, parallel, bcastIdxs):
                     on.
     :arg parallel:  If ``True``, each variable group is processed in parallel.
                     Otherwise they are processed sequentially.
-    :arg bcastIDxs: Indices for broadcast arguments, for each variable group.
     """
 
     results = []
 
-    if bcastIdxs is None:
-        bcastIdxs = [None] * len(vids)
+    def filterMissing(vids):
+        """Takes a list of variable IDs and removes those that are not
+        present in the data set, emitting a warning for each ID that
+        is removed.
+        """
+        if not proc.filterMissing:
+            return vids
+        filtered = []
+        for vid in vids:
+            if dtable.present(vid):
+                filtered.append(vid)
+            else:
+                log.warning('Process %s refers to missing variable %u! '
+                            '(%s)', proc.name, vid, proc.processString)
+        return filtered
 
     # run process serially
     if not parallel:
         for vg in vids:
+            vg = filterMissing(vg)
             results.append(proc.run(dtable, vg))
 
     # or run in parallel across vid groups
     else:
         with dtable.pool() as pool:
 
+            # Note. This code is horrible for a number of
+            # reasons, including that parallelisation in
+            # older versions of FUNPACK worked differently,
+            # and for the sake of preserving backwards
+            # compatibility with resepct to the use of the
+            # argument broadcast feature (although I
+            # seriously doubt that anybody is even using
+            # broadcasting).
+
             # gather all variables required by this process -
             # the ones which are specified in the processing
             # table, along with any auxillary ones speciified
-            # as arguments to the process.
-            allvids    = [vg + proc.auxillaryVariables(bi)
-                          for vg, bi in zip(vids, bcastIdxs)]
+            # as arguments to the process. "allvids" is used
+            # to creae the sub-table, and "vids" is the list
+            # of vids that we ask the processing function to
+            # process.
+            unfiltered = vids
+            vids       = []
+            allvids    = []
+            bcastIdxs  = []
+
+            for i, vg in enumerate(unfiltered):
+                vg      = filterMissing(vg)
+                auxvids = filterMissing(proc.auxillaryVariables(i))
+                if len(vg) > 0:
+                    vids     .append(vg)
+                    allvids  .append(vg + auxvids)
+                    bcastIdxs.append(i)
 
             # generate a subtable for each variable group -
             # this is to minimise the amount of data that
@@ -449,14 +477,15 @@ class Process:
     """
 
 
-    def __init__(self, ptype, name, args, kwargs):
+    def __init__(self, ptype, name, args, kwargs, procstr):
         """Create a ``Process``.
 
-        :arg ptype:  Process type - either ``cleaner`` or ``processor``
-                     (see the :mod:`.custom` module).
-        :arg name:   Process name
-        :arg args:   Positional arguments to pass to the process function.
-        :arg kwargs: Keyword arguments to pass to the process function.
+        :arg ptype:   Process type - either ``cleaner`` or ``processor``
+                      (see the :mod:`.custom` module).
+        :arg name:    Process name
+        :arg args:    Positional arguments to pass to the process function.
+        :arg kwargs:  Keyword arguments to pass to the process function.
+        :arg procstr: Input string containing the process specification.
 
         Any keyword arguments which begin with ``'broadcast_'`` are separated
         out the other keyword arguments - see the :meth:`run` method for more
@@ -476,12 +505,13 @@ class Process:
         # cleaner functions are not
         # defined in processing_functions,
         # so in this case func will be None.
-        self.__ptype       = ptype
-        self.__name        = name
-        self.__args        = args
-        self.__kwargs      = normalKwargs
-        self.__bcastKwargs = bcastKwargs
-        self.__metaproc    = normalKwargs.pop('metaproc', None)
+        self.__ptype         = ptype
+        self.__name          = name
+        self.__args          = args
+        self.__kwargs        = normalKwargs
+        self.__procstr       = procstr
+        self.__bcastKwargs   = bcastKwargs
+        self.__metaproc      = normalKwargs.pop('metaproc', None)
 
 
     def __repr__(self):
@@ -515,6 +545,14 @@ class Process:
         return self.__kwargs
 
 
+    @property
+    def processString(self):
+        """Returns the original string, from the processing table/
+        command-line, which defines this ``Process``.
+        """
+        return self.__procstr
+
+
     @property
     def broadcastKwargs(self):
         """Returns the keyword arguments for this ``Process`` which
@@ -524,6 +562,22 @@ class Process:
         return self.__bcastKwargs
 
 
+    @property
+    def filterMissing(self):
+        """Return ``True`` if this processing function expects that the list of
+        variable IDs which it is given will not contain the IDs of variables
+        which are not present in the data.
+
+        This property is set via a ``filterMissing`` argument passed to the
+        processor decorator function. Its default value is ``True``.
+
+        Note: This is a hack which is only used by the
+        :func:`.processing_functions.binariseCategorical` function, and which
+        is in place because that function used to be parallelised differently.
+        """
+        return custom.args(self.__ptype, self.__name).get('filterMissing', True)
+
+
     def auxillaryVariables(self, broadcastIndex=None):
         """Returns a list of "auxillary" variables for this process. Auxillary
         variables are variables which a process is not being applied to, but
@@ -533,6 +587,10 @@ class Process:
         The names of any arguments which contain auxillary variables are
         specified via the ``auxvids`` argument to the processor decorator
         function.
+
+        Note: This is a hack which is only used by the
+        :func:`.processing_functions.binariseCategorical` function, and which
+        is in place because that function used to be parallelised differently.
         """
 
         auxargs = custom.args(self.__ptype, self.__name).get('auxvids', [])
@@ -648,7 +706,7 @@ def parseProcesses(procs, ptype):
         if not custom.exists(ptype, name):
             raise NoSuchProcessError(name)
 
-        return Process(ptype, name, args, kwargs)
+        return Process(ptype, name, args, kwargs, procs)
 
     parser = pp.delimitedList(makeParser().setParseAction(makeProcess))
 
