@@ -76,6 +76,8 @@ from . import processing_functions_core as core
 from . import                              util
 from . import                              custom
 from . import                              datatable
+from . import                              loadtables
+
 
 log = logging.getLogger(__name__)
 
@@ -150,17 +152,29 @@ def removeIfSparse(
 
 
 @custom.processor()
-def removeIfRedundant(dtable, vids, corrthres, nathres=None, pairwise=False):
+def removeIfRedundant(dtable       : datatable.DataTable,
+                      vids         : List[int],
+                      corrthres    : float,
+                      nathres      : Optional[float] = None,
+                      pairwise     : Optional[bool]  = False,
+                      skipUnknowns : Optional[bool]  = False):
     """removeIfRedundant(corrthres, [nathres], [pairwise])
     Removes columns deemed to be redundant.
 
     Removes columns from the variables in ``vids`` if they are redundant.
 
     Redundancy is determined by calculating the correlation between pairs
-    of columns - see the :func:`.isRedundant` function.
+    of columns using `corrthres` and `nathres` - see the :func:`.isRedundant`
+    function.
 
     If ``pairwise`` is ``True``, an alternative implementation is used which
     may be faster on data sets with high missingness correlation.
+
+    :arg pairwise:     Use alternative pairwise implementation.
+
+    :arg skipUnknowns: If ``True``, columns which are deemed to be redundant
+                       with respect to an unknown or uncategorised column
+                       are *not* dropped.
     """
 
     # Ignore non-numeric columns
@@ -169,19 +183,37 @@ def removeIfRedundant(dtable, vids, corrthres, nathres=None, pairwise=False):
     colnames = [c.name for c in cols]
     data     = dtable[:, colnames]
 
-    if pairwise:
-        redundant = _pairwiseRemoveIfRedundant(
-            dtable, data, corrthres, nathres)
-    else:
-        redundant = _removeIfRedundant(dtable, data, corrthres, nathres)
+    with np.errstate(divide='ignore'):
+        if pairwise:
+            redundant = _pairwiseRemoveIfRedundant(
+                dtable, data, corrthres, nathres)
+        else:
+            redundant = _removeIfRedundant(
+                dtable, data, corrthres, nathres)
 
     redundant = util.dedup(sorted(redundant))
 
-    if len(redundant) > 0:
-        log.debug('Dropping %u redundant columns: %s ...',
-                  len(redundant), redundant[:5])
+    if skipUnknowns:
+        copy = []
+        for idxa, idxb in redundant:
+            colb = cols[idxa]
+            bvid = colb.vid
+            cats = loadtables.variableCategories(dtable.cattable, [bvid])[bvid]
 
-    return [cols[r] for r in redundant]
+            if 'unknown' in cats or 'uncategorised' in cats:
+                namea = colnames[idxa]
+                nameb = colnames[idxb]
+                log.debug('Column %s is redundant with %s, but %s is '
+                          'unknown / uncategorised, so %s will not be '
+                          'dropped', namea, nameb, nameb, namea)
+            else:
+                copy.append((idxa, idxb))
+        redundant = copy
+
+    if len(redundant) > 0:
+        log.debug('Dropping %u redundant columns', len(redundant))
+
+    return [cols[r[0]] for r in redundant]
 
 
 def _removeIfRedundant(dtable, data, corrthres, nathres=None):
@@ -193,7 +225,9 @@ def _removeIfRedundant(dtable, data, corrthres, nathres=None):
     :arg corrthres: Correlation threshold - see :func:`.redundantColumns`.
     :arg nathres:   Missingness correlation threshold - see
                     :func:`.redundantColumns`.
-    :returns:       A sequence of indices denoting the redundant columns.
+    :returns:       Sequence of tuples of column indices, where each tuple
+                    ``(a, b)`` indicates that column ``a`` is redundant with
+                    respect to column ``b``.
     """
 
     return core.matrixRedundantColumns(data, corrthres, nathres)
@@ -208,7 +242,9 @@ def _pairwiseRemoveIfRedundant(dtable, data, corrthres, nathres=None):
     :arg corrthres: Correlation threshold - see :func:`.redundantColumns`.
     :arg nathres:   Missingness correlation threshold - see
                     :func:`.redundantColumns`.
-    :returns:       A sequence of indices denoting the redundant columns.
+    :returns:       Sequence of tuples of column indices, where each tuple
+                    ``(a, b)`` indicates that column ``a`` is redundant with
+                    respect to column ``b``.
     """
 
     ncols = len(data.columns)
@@ -409,7 +445,7 @@ def binariseCategorical(dtable,
 
     # if take is a single vid or None,
     # we turn it into [take] * len(vids)
-    if not isinstance(take, collections.Sequence):
+    if not isinstance(take, collections.abc.Sequence):
         take = [take] * len(vids)
 
     if len(take) != len(vids):
