@@ -155,7 +155,7 @@ def isSparse(
             return True, 'minpres', npresent
 
     # stddev is not large enough (for
-    # numerical/categorical types
+    # numerical/categorical types)
     if isnumeric and minstd is not None:
         std = (present - present.mean()).std()
         if std <= minstd:
@@ -173,8 +173,18 @@ def isSparse(
         # maxcat - one category is too dominant
         uniqvals   = pd.unique(present)
         uniqcounts = [sum(present == u) for u in uniqvals]
-        nmincat    = min(uniqcounts)
-        nmaxcat    = max(uniqcounts)
+
+        # Column is empty - force the test to fail
+        if len(uniqcounts) == 0:
+            if mincat is not None:
+                mincat  = 1
+                nmincat = 0
+            if maxcat is not None:
+                maxcat  = 0
+                nmaxcat = 0
+        else:
+            nmincat = min(uniqcounts)
+            nmaxcat = max(uniqcounts)
 
         if mincat is not None:
             if nmincat < mincat:
@@ -238,7 +248,7 @@ def pairwiseRedundantColumns(
         colpairs  : np.ndarray,
         corrthres : float,
         token     : Optional[str] = None
-) -> List[int]:
+) -> List[Tuple[int, int]]:
     """Identifies redundant columns based on their correlation with each
     other by comparing each pair of columns one by one.
 
@@ -252,7 +262,9 @@ def pairwiseRedundantColumns(
 
     :arg token:     Identifier string for log messages.
 
-    :returns:       Sequence of redundant column indices.
+    :returns:       Sequence of tuples of column indices, where each tuple
+                    ``(a, b)`` indicates that column ``a`` is redundant with
+                    respect to column ``b``.
     """
 
     if len(colpairs) == 0:
@@ -261,7 +273,7 @@ def pairwiseRedundantColumns(
     if token is None: token = ''
     else:             token = '[{}] '.format(token)
 
-    redundant = set()
+    redundant = {}
     nacounts  = data.isna().sum(axis=0).to_numpy()
 
     # calculate correlation between column pairs
@@ -282,17 +294,18 @@ def pairwiseRedundantColumns(
             if nacounts[coli] > nacounts[colj]: drop, keep = coli, colj
             else:                               drop, keep = colj, coli
 
-            log.debug('%sColumn %s is redundant (correlation with %s: %f)',
-                      token, data.columns[drop], data.columns[keep], corr)
-            redundant.add(drop)
+            if drop not in redundant:
+                log.debug('%sColumn %s is redundant (correlation with %s: %f)',
+                          token, data.columns[drop], data.columns[keep], corr)
+                redundant[drop] = keep
 
-    return list(redundant)
+    return list(redundant.items())
 
 
 def matrixRedundantColumns(
         data      : pd.DataFrame,
         corrthres : float,
-        nathres   : Optional[float] = None) -> np.ndarray:
+        nathres   : Optional[float] = None) -> List[Tuple[int, int]]:
     """Identifies redundant columns based on their correlation with each
     other using dot products to calculate a correlation matrix.
 
@@ -306,13 +319,19 @@ def matrixRedundantColumns(
                     ``corrthres`` *and* a missing-value correlation greater
                     than ``nathres`` to be identified as redundant.
 
-    :returns:       Sequence of redundant column indices.
+    :returns:       Sequence of tuples of column indices, where each tuple
+                    ``(a, b)`` indicates that column ``a`` is redundant with
+                    respect to column ``b``.
     """
 
     if len(data.columns) < 2:
         return []
 
-    data = data.to_numpy(dtype=np.float32, copy=True)
+    # Keep a copy of the column names for logging.
+    # Create a 2D matrix containing all data, using
+    # float32 to limit memory consumption.
+    columns = data.columns
+    data    = data.to_numpy(dtype=np.float32, copy=True)
 
     namask   = np.isnan(data)
     nacounts = namask.sum(axis=0)
@@ -362,7 +381,9 @@ def matrixRedundantColumns(
     # correlation coefficient
     coef = (xy - n * mx * my) / (sx * sy)
 
-    # ignore nans/infs, binarise
+    # ignore nans/infs, binarise. Keep a copy
+    # of the raw correlations for logging.
+    rawcoef                  = coef
     coef[~np.isfinite(coef)] = 0
     coef                     = np.abs(coef) > corrthres
 
@@ -380,14 +401,36 @@ def matrixRedundantColumns(
 
     # for each correlated pair, we flag the
     # one with more missing values as redundant
-    def mostna(coli, colj):
-        if nacounts[coli] > nacounts[colj]: return coli
-        else:                               return colj
+    def correlatedPairs(coli, colj):
+        if nacounts[coli] > nacounts[colj]: drop, keep = coli, colj
+        else:                               drop, keep = colj, coli
 
-    mostna    = np.vectorize(mostna, [np.uint32])
-    redundant = mostna(colpairs[0], colpairs[1])
+        # coef is only the upper triangle
+        if coli <= colj: corr = rawcoef[coli, colj]
+        else:            corr = rawcoef[colj, coli]
 
-    return np.unique(redundant)
+        log.debug('Column %s is redundant (correlation with %s: %0.6f)',
+                  columns[drop], columns[keep], corr)
+
+        return drop, keep
+
+    # Generate a sequence of pairs, where the first
+    # element is the column to drop, and the second
+    # element is the column it is redundant w.r.t.
+    correlatedPairs = np.vectorize(correlatedPairs, [np.uint32, np.uint32])
+    colsa, colsb    = correlatedPairs(colpairs[0], colpairs[1])
+
+    if len(colsa) == 0:
+        return []
+
+    # Return only one pair for each column
+    # that is to be dropped (the first
+    # pair in the natural column ordering).
+    idxs      = np.unique(colsa, return_index=True)[1]
+    colsa     = colsa[idxs]
+    colsb     = colsb[idxs]
+
+    return list(zip(colsa, colsb))
 
 
 def binariseCategorical(
@@ -469,6 +512,9 @@ def binariseCategorical(
     uniq = [v for v, c in uniq.items() if c >= minpres]
     uniq = list(sorted(uniq))
 
+    if len(uniq) == 0:
+        return np.zeros((len(data), 0)), np.zeros((0,))
+
     log.debug('%sCounted %u unique values [minpres: %u]',
               token, len(uniq), minpres)
 
