@@ -289,3 +289,45 @@ def test_Expresssion_non_numeric():
 
         assert len(result) == len(expected)
         assert all([bool(r) == bool(e) for r, e in zip(result, expected)])
+
+
+def test_Expresssion_datetime():
+    testdata = tw.dedent("""
+    eid,10,20
+    1,2020-01-01,2020-01-01 01:00:00
+    2,2020-02-01,2020-01-01 02:00:00
+    3,2020-03-01,2020-01-01 03:00:00
+    4,2020-04-01,2020-01-01 04:00:00
+    """)
+
+    test_cols = {
+        10 : '10',
+        20 : '20',
+    }
+
+    expr_tests = [
+        ('v10 <= 2020-03-01',          [1, 1, 1, 0]),
+        ('v10 == 2020-01-01',          [1, 0, 0, 0]),
+        ('v10 >  2020-02-01',          [0, 0, 1, 1]),
+        ('v20 == 2020-01-01 02:00:00', [0, 1, 0, 0]),
+        ('v20 <  2020-01-01 03:00:00', [1, 1, 0, 0]),
+        ('v20 >= 2020-01-01 02:00:00', [0, 1, 1, 1]),
+    ]
+
+    data =  pd.read_csv(io.StringIO(testdata),
+                        sep=',',
+                        parse_dates=['10', '20']).set_index('eid')
+
+    for expr, expected in expr_tests:
+
+        if expected == 'error':
+            with pytest.raises(pp.ParseException):
+                e = expression.Expression(expr)
+            continue
+        else:
+            e = expression.Expression(expr)
+
+        result = e.evaluate(data, test_cols)
+
+        assert len(result) == len(expected)
+        assert all([bool(r) == bool(e) for r, e in zip(result, expected)])
