@@ -380,7 +380,8 @@ def test_encoding():
     latin1_val = '\xa1\xa2\xa3'
     utf8_val   = '\u1F610\u1F632\u1F640'
 
-    vartable, proctable, cattable = gen_tables(range(1, 2))[:3]
+    vartable, proctable, cattable = gen_tables(
+        range(1, 2), vtypes={1:'text'})[:3]
 
     with tempdir():
 
