@@ -1221,11 +1221,11 @@ def test_main_column_patterns_and_variables():
 def test_main_columns_from_file():
     data = tw.dedent("""
     eid,a,b,c,d1,d2,e
-    1,10,100,1000,10000,100000,1000000
-    2,20,200,2000,20000,200000,2000000
-    3,30,300,3000,30000,300000,3000000
-    4,40,400,4000,40000,400000,4000000
-    5,50,500,5000,50000,500000,5000000
+    1,10.0,100.0,1000.0,10000.0,100000.0,1000000.0
+    2,20.0,200.0,2000.0,20000.0,200000.0,2000000.0
+    3,30.0,300.0,3000.0,30000.0,300000.0,3000000.0
+    4,40.0,400.0,4000.0,40000.0,400000.0,4000000.0
+    5,50.0,500.0,5000.0,50000.0,500000.0,5000000.0
     """).strip()
 
 
