@@ -18,6 +18,7 @@ import numpy as np
 import pandas as pd
 
 import funpack.main       as main
+import funpack.util       as util
 import funpack.icd10      as icd10
 import funpack.custom     as custom
 import funpack.expression as expression
@@ -63,8 +64,10 @@ def test_main_icd10():
         with open('data.tsv',  'wt') as f: f.write(data)
 
         with mock.patch('funpack.hierarchy.getHierarchyFilePath',
-                        return_value='icd10.tsv'):
-            main.main('-cl 1 codeToNumeric(\'icd10\') '
+                        return_value='icd10.tsv'), \
+            mock.patch.dict('funpack.util.DATA_TYPES',
+                             {util.CTYPES.unknown : str}):
+            main.main('-nb -cl 1 codeToNumeric(\'icd10\') '
                       '-imf icd10_mappings.tsv out.tsv data.tsv'
                       .split())
 
