@@ -198,6 +198,26 @@ def test_removeIfRedundant_parallel():
         assert parrem == seqrem
 
 
+@patch_logging
+def test_removeIfRedundant_skipUnknowns():
+    series1 = np.sin(np.linspace(0, np.pi * 6, 100))
+    series2 = series1 + np.random.random(100)
+    series3 = series2
+
+    corr    = pd.Series(series1).corr(pd.Series(series2))
+    dtable  = gen_DataTable([series1, series2, series3])
+    # 1 will be in uncategorised
+    dtable.cattable['some category'] = [2, 3]
+    remove = pfns.removeIfRedundant(dtable, [1, 2, 3], corr * 0.9)
+    remove = [r.name for r in remove]
+    assert remove == ['2-0.0', '3-0.0']
+
+    remove = pfns.removeIfRedundant(dtable, [1, 2, 3], corr * 0.9,
+                                    skipUnknowns=True)
+    remove = [r.name for r in remove]
+    assert remove == []
+
+
 @patch_logging
 def test_binariseCateorical():
 
@@ -364,7 +384,8 @@ def test_binariseCategorical_nonnumeric():
             for i, (v1, v2) in enumerate(zip(*data)):
                 f.write('{}, {}, {}\n'.format(i + 1, v1, v2))
 
-        vartable, proctable, cattable = gen_tables([1])[:3]
+        vartable, proctable, cattable = gen_tables(
+            [1], vtypes={1 : 'text'})[:3]
         finfo = fileinfo.FileInfo('data.txt')
         dt, _ = importing.importData(finfo, vartable, proctable, cattable)
 
@@ -498,7 +519,8 @@ def test_binariseCategorical_take():
                 f.write(cols + '\n')
                 f.write(data)
 
-            vartable, proctable, cattable = gen_tables([1, 2])[:3]
+            vartable, proctable, cattable = gen_tables(
+                [1, 2], vtypes={1 : 'text'})[:3]
             finfo = fileinfo.FileInfo('data.txt')
             dt, _ = importing.importData(finfo, vartable, proctable, cattable)
 
@@ -576,7 +598,8 @@ def test_expandCompound():
                 f.write(str(i + 1) + '\t' + ','.join(map(str, data[i])))
                 f.write('\n')
 
-        vartable, proctable, cattable = gen_tables([1])[:3]
+        vartable, proctable, cattable = gen_tables(
+            [1], vtypes={1 : 'text'})[:3]
         finfo = fileinfo.FileInfo('data.txt')
         dt, _ = importing.importData(finfo, vartable, proctable, cattable)
 
