@@ -87,7 +87,8 @@ def parseColumnName(name):
     where ``variable`` and ``visit`` are integers. ``instance`` is typically
     also an integer, but non-numeric values for ``instance`` are
     accepted. This (and the second form above) is to allow parsing of derived
-    columns (see e.g. the :func:`.binariseCategorical` processing function).
+    columns (see e.g. the :func:`.processing_functions.binariseCategorical`
+    processing function).
 
     Some variables have the form::
 
@@ -117,6 +118,7 @@ def parseColumnName(name):
 
     :arg name: Column name
     :returns:  A tuple containing:
+
                 - variable ID
                 - visit number
                 - instance (may be an integer or a string)
